%Make sure that we change path to the folder with the main file (the 'SAEM'
%folder)
addpath('./UpdatedFunctions2/');
addpath('./dxFileswithGetXTildFreeIC/');


plot_flag = 1;
noRuns = 1;
batch = 1; %Running simulations in batches so keeping track of this, but can be removed throughout.
trueInit = 1; %1 = x0~N(mu, Sigma_mu), ran for 50 time points, demeaned at t = 50, then retain from 51th time point and beyond
              %2 = fixed init, x0 = [start1, start2]
%Other files of interest
             % FitFreeICwb: Basically this file, but written as a function;
                  %estimates mu as part of theta_f and Sigmamu as part of Sigmab (freeIC with b)
                  %This specification should be able to handle both
                  %trueInit as special cases. See how it performs compared
                  %to the other two fitInit conditions
             % FitFreeICnob: free IC (estimates start1 and start2) with no b.
                  %IC is misspecified if trueInit = 1, see if performance
                  %is compromised when compared to trueInit = 2, fitInit =
                  %3.
             % FitFixedIC: fixed IC (fix initial condition at start1 and start2--IC
                  %is misspecified if trueInit = 1)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
warning off all;

%for kk=1:noRuns, %Loop for Monte Carlo runs. Uncomment this and the
%closing command ("end" line at the very end of this script) to run Monte
%Carlo simulations.
kk=1;
rand1 = round(cputime*batch*((kk)+(kk-1)*noRuns)); %Important to have different seeds if
%running multiple jobs on the cluster simultaneously
rng(rand1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (1) Generate data 
% genDataFreeICwb will generate data with
% subject-specific time intervals. 
% minDelt is the minimum delta_t.
% maxDelt is the maximum delta_t.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
batch2 = int2str(batch);

filenametrueb = strcat('TrueInit',trueInit,'truebG',batch2,'.txt');
filenameY = strcat('TrueInit',trueInit,'Y',batch2,'.txt');

%plot_flag = a binary flag (0=no, 1=yes) to turn on plotting of the
%generated data.
[InfDS0] = genDataFreeICwb(plot_flag,trueInit,filenametrueb,filenameY,batch,kk);

%Read in data again just to check
%Data are in the order of batch, kk, trueInit, Elapsed time, DVs, Covariates
Y = dlmread(filenameY);
  
InfDS = {};
InfDS.Nx = 7;%Number of latent variables with theta_fi stacked in.
InfDS.NxState=2; %Number of latent variables (x_1, x_2)
InfDS.Ny = 3;%Number of manifest variables
InfDS.Nbeta = 0; %Size of beta (fixed effects parameters for elements in theta_i); Set to 0 here
                 %because data generation was done by sticking the fixed
                 %effects parameters into the latent state vector. Later
                 %during model fitting, this will be set to 5 corresponding
                 %to beta = [\zeta_0 \zeta_1 \zeta_2 \mu_{x1} and
                 %\mu_{x2}]'
InfDS.Ntheta = 3; %Size of theta_fi: dynamic parameters with subject-specific parameters (fixed and random effects)
InfDS.Nbetax = InfDS.Nx-InfDS.NxState; %Number of fixed effects parameters that are in the state vector
InfDS.Nb = 3;%Number of random effects (b_zeta, b_x1, b_x2)
InfDS.Nmu = 3; %Number of intercept parameters (mu_1, mu_2, and mu_3)
InfDS.NLambda = 2; %Number of parameters in factor loading matrix
InfDS.Nbpar = 4; %Number of parameters in random effects variance-covariance matrix (Eq 15)
InfDS.Nu = 2; %Number of covariates predicting the person-specific theta_fi
%InfDS.N = 1; %Not used any more. At one point was doing multiple chains of
% MCMC samples concurrently.


InfDS.Z = eye(InfDS.Nb); %Random effects design matrix
InfDS.Nbetax = InfDS.Nx-InfDS.NxState; %= 5, Fixed effects parameters that are to be put into state vector
InfDS.G=eye(InfDS.Nx); %Design matrix for process noise terms - just set to identity matrix
InfDS.Npar0 = InfDS.Nmu + InfDS.NLambda + InfDS.Ny + InfDS.Nbpar; %Total number of parameters
period = .8;
InfDS.omega = (2*pi/period)^2; %Fixed and not estimated -(2pi/gamma)^2
InfDS.totalT = 300; %Total number of time points - Only use in data generation, I think. Check estimation scripts, switch all instances to InfDS.allT
InfDS.Nsubj = 200; %Total number of subjects
%InfDS.tStart= 50; %Not used. Number of initial time points to throw away in data generation
InfDS.maxT =  max(Y(:,4)); %max(T_{i,T}) - In applications get from observed data
InfDS.delt =  InfDS0.delt;%0.0050= %minimum integration time step
InfDS.tspan = round2(0:InfDS.delt:InfDS.maxT,InfDS.delt); % :-) Same as InfDS.fulltime. Set InfDS.fulltime to this if it appears in estimation functions
InfDS.fulldt=cell(InfDS.Nsubj); 
InfDS.Y = cell(InfDS.Nsubj);
InfDS.timeDiscrete = cell(InfDS.Nsubj);
InfDS.allT = repmat(InfDS.totalT,1,InfDS.Nsubj); %A vector holding InfDS.totalT for all subjects

%H holds the design matrix for the fixed effects so we need to put all the
%subject-specific covariates, u_{1i} and u_{2i} in here.
InfDS.H = zeros(InfDS.Ntheta*InfDS.Nsubj, InfDS.Nbetax);
InfDS.U1 = zeros(InfDS.Nsubj, InfDS.Nu);


for i = 1:InfDS.Nsubj,
    InfDS.U1(i,:) = Y((1+(i-1)*InfDS.totalT),(InfDS.Ny+5):(InfDS.Ny+4+InfDS.Nu)); % :-) Get from data
    InfDS.H((1+(i-1)*InfDS.Ntheta):(i*InfDS.Ntheta),:) = [1 InfDS.U1(i,1) InfDS.U1(i,2) 0 0;
                                                          0 0 0 1 0;
                                                          0 0 0 0 1];
    InfDS.Y{i} = (Y((1+(i-1)*InfDS.totalT):(i*InfDS.totalT),5:(InfDS.Ny+4)))'; % observed DV data
    InfDS.timeDiscrete{i} = round2(Y((1+(i-1)*InfDS.totalT):(i*InfDS.totalT),4),InfDS.delt);% observed data.

    InfDS.lens(i)=length(InfDS.tspan);%length(T_{i,T}) including interpolation time steps
    [tf,loc]=ismember(InfDS.timeDiscrete{i},InfDS.tspan);
    InfDS.tobs{i} = loc(loc>0);% :-) Check if still used later. Index for positions of elements in tspan that are observed in the data, e.g., 1, 2, 3, 5, 6
    InfDS.Deltat{i} = [InfDS.delt;  diff(InfDS.timeDiscrete{i})];%Observed delta_t
    InfDS.fulldt{i}=[InfDS.delt diff(InfDS.tspan)];%diff(full_time{i});
end

InfDS.allDelta = reshape(cell2mat(InfDS.Deltat),1,InfDS.Nsubj*InfDS.totalT);% :-) InfDS.Deltat as a 1 x (InfDS.Nsubj*InfDS.totalT) vector
InfDS.Tfilter = ones(1,InfDS.Nsubj*InfDS.totalT); % :-) Created by Zhaohua? Not sure why this is needed

InfDS.odefn    = @dynfunParIC;      % File name for ODE functions of the Van der Pol - parameters in state vector
InfDS.Jdyn  = @dfdxParIC; %File name for Jacobian matrix of state variables. Not used in data gen but needed for ekf later.
InfDS.yfun = @VDPMeas; %Measurement functions

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (2) Model fitting begins here
% Generally model fitting unfolds over a few steps: 
% (2a) We first define an expanded latent variable vector with both the
%      state variables and beta in it. Run EKF to jointly estimate these.
% (2b) Then set the beta parameters to these EKF estimates, Run drawb with
%      these starting parameter values. Use these as starting values for b.
% (2c) Run EKF again to jointly estimate state variables and beta
% (3) Run the SAEM with these starting estimates.
% (4) Wrap up and write out estimates
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

tic; %Time counter starts here


%Check genDataFreeICwb.m for details of how time steps are set up.
%InfDS.delt is the fixed time step used in numerical solutions of the ODEs
%InfDS.tspan holds a list of time intervals from 0 to InfDS.maxT, with
%increments InfDS.delt.
%
%Here tmp basiscally is a matrix of 0s and 1s indicating whether a
%particular time point in InfDS.tspan is observed.

tmp = zeros(length(InfDS.tspan),InfDS.Nsubj);
for i= 1:InfDS.Nsubj
for t = 1:length(InfDS.tspan)
	if min(abs(InfDS.tobs{i}-t)) == 0 
		tmp(t,i) = 1;
	end
end
end
InfDS.tidx = logical(tmp);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (2a) Set up starting values and control parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Specify output file names
filenamePar = strcat('TrueInit',trueInit,'parG',batch2,'.txt');
filenameSE = strcat('TrueInit',trueInit,'SEG',batch2,'.txt');
filenamebhat = strcat('TrueInit',trueInit,'bhatG',batch2,'.txt');
filenamebhat2 = strcat('TrueInit',trueInit,'bhat2G',batch2,'.txt');
filenameconv = strcat('TrueInit',trueInit,'convG',batch2,'.txt');

%True values
%mu1 = 0;
%mu2 = 0;
%mu3 = 0;
%vsigma1 = .5;
%vsigma2 = .5;
%vsigma3 = .5;
%truepars = [mu1, mu2, mu3, lambda21,lambda31,vsigma1,vsigma2,vsigma3,vsigmab1,vsigmab2,vsigmab3,v23];

%Starting values for the parameters shown above in truepars.
%Assuming no random effects so vsigmab1-vsiamgb3, v23 not used and set to NaN. 
InfDS.startpars = [0,0,0,1,1,log(1), log(1), log(1) NaN NaN NaN NaN];

%Set up control parameters for SAEM
InfDS.gainpara = .6; InfDS.gainparb = 3; %a1 & a2 in SAEM gain constant
InfDS.gainpara1 = .9; InfDS.gainparb1 = 1; % a1 & a2 in SAEM second stage - gives smaller gain constant
InfDS.MAXITER =200; %Maximum number of iterations
InfDS.maxIterStage1 = 100; %Maximum number of iterations for stage 1
switchFlag = 0; %Flag for whether the SAEM algorithm has switched to stage 2
InfDS.isInfo=1; %Flag for whether computation of information matrix is needed - check: not sure if this is ever set to 0.



NthetaInit = size(InfDS.H,1)/InfDS.Nsubj;
InfDS.P0 = 1e-7*eye(InfDS.Nx);
InfDS.P0(3,3) = 1;
InfDS.P0(4,4) = .1;
InfDS.P0(5,5) = .1;
InfDS.P0(6,6) = 1;
InfDS.P0(7,7) = 1;
    
%Set to arbitrary values
Q = [1,1,1,.2]; %Starting values (untransformed scale) of all variances in Sigmab, then cov(2,3)

%%%%%
%Sigmab looks like this currently with the LDL' decomposition:
%[      exp(par1),          0,                   0]
%[      0,          exp(par2),      par4*exp(par2)]
%[      0,          par4*exp(par2), exp(par2)*par4^2 + exp(par3)]
InfDS.startpars(9:12) = [log(Q(1)), log(Q(2)),  log(Q(3) - Q(4)^2/Q(2)), Q(4)/Q(2) ];
InfDS.par(9:12) =  [log(Q(1)), log(Q(2)),  log(Q(3) - Q(4)^2/Q(2)), Q(4)/Q(2) ];
InfDS.par = InfDS.startpars;
    
%startb =  round2((chol(InfDS.Sigmab)*randn(InfDS.Nb,InfDS.Nsubj))',10e-4);
startb =  round2((.001*eye(InfDS.Nb)*randn(InfDS.Nb,InfDS.Nsubj))',10e-4);
InfDS.b = startb;
InfDS.pNoiseadaptParams = {};

InfDS.sy = zeros(length(InfDS.par),1);
InfDS.ES = zeros(length(InfDS.par),length(InfDS.par));
InfDS.EI = zeros(length(InfDS.par),length(InfDS.par));
InfDS.Iytild = zeros(length(InfDS.par),length(InfDS.par));
InfDS.Iy = zeros(length(InfDS.par),length(InfDS.par));
InfDS.lowBound = NaN*ones(length(InfDS.par),1);
InfDS.upBound = NaN*ones(length(InfDS.par),1);
InfDS.lowBound(4:5) = 10e-8;
InfDS.upBound(4:5) = 10;
InfDS.lowBound(6:11) = log(10e-7);
InfDS.upBound(6:11) = log(10);
InfDS.N = 1; freeIC = 0;%FreeIC always = 0 in this EKF part

%[tpOld] = ekfContinuous10(InfDS);

disp '(2a) Get better starting values for the dynamic parameters';

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Get better starting values for the dynamic parameters in beta
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %First find decent estimates for initial conditions
    InfDS.meanY = cell(InfDS.Nsubj);
    y01 = zeros(InfDS.Nsubj,InfDS.NxState);
    meanY = 0; meanDY = 0;
    for ss=1:InfDS.Nsubj
    datnow = InfDS.Y{ss};
    InfDS.meanY{ss} = mean(datnow,1);
    %Get data-driven starting values for initial level and first derivative
    y01(ss,1) = InfDS.meanY{ss}(1);    
    y01(ss,2) = InfDS.meanY{ss}(2) - InfDS.meanY{ss}(1); 
    meanY = meanY + InfDS.meanY{ss}(1)/InfDS.Nsubj;
    meanDY = meanDY + y01(ss,2)/InfDS.Nsubj;
    InfDS.b(ss,2) = y01(ss,1) - meanY;
    end;
    
    InfDS.y0 = repmat([0 0 2 .1 .1 meanY meanDY],InfDS.Nsubj,1); 
     
    gmm0=1;sgnTH0 = zeros(length(InfDS.par),1);stage=0; 
    for k=1:10
    [InfDS] = setParsEKFFreeICwb(InfDS);
    %ekfContinuous4  - runs the extended Kalman filter to jointly estimate
    %parameters and latent variables 
    [InfDS,negloglik,Xh_ekf, P_ekf, Xh_pred, PPred, inov, Finv, ~,chiX,tp, xk1,xk2] = ekfContinuous4(InfDS,-2);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    [score,infoMat] =  getScoreInfoY_tobs_opt(InfDS,stage,k,freeIC);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % SAEM algorithm
    [InfDS, stop0, ~, gmm0] = saem(InfDS,gmm0,0,sgnTH0, score,score*score',infoMat,{});
    InfDS = getXtildIC3(InfDS,0,freeIC); %Get updated Xtilde
    %Use elements in x1 as estimates for mu_{x1} and mu_{x2}
    x1 = mean(reshape(Xh_ekf(InfDS.NxState+1:InfDS.Nx,:,InfDS.totalT),InfDS.Nbetax,(1)*InfDS.Nsubj),2);
    InfDS.y0 = repmat([0,0 x1'],InfDS.Nsubj,1);
        
    %Toward the end of the 10 iterations set b_{x1i} and b_{x2i} in InfDS.b to the deviations in
    %each individual's filtered estimates of x_{i1}(t_{i1}) and x_{i2}(t_{i1}) from 
    %x1
        for ss=1:InfDS.Nsubj
            InfDS.b(ss,2) = Xh_ekf(6,ss,InfDS.allT(ss)) - x1(4);
            if k==5||k==9, %Estimates for initial condition for first derivatives pretty unstable, don't reset too much.
            InfDS.b(ss,3) = Xh_ekf(7,ss,InfDS.allT(ss)) - x1(5); 
            end
        end
        
    end   
    
%x1 now holds estimaes for [zetaSBP beta1 beta2 mu_{x1} mu_{x2}]' (see Eq.
%14). The followings are the true values for these parameters. Not bad for
%starting values. Some biases in mu_{x1} and mu_{x2} are not too bad.

%zetaSBP = 3;
%beta1 = .5;
%beta2 = .5;
%mu_x1 = start1 = 1; mu_{x2} = start2 = 1.
temp0 = x1; %Use these as starting values for the parameters in beta- refine again next
%x1 =
%
%    2.9004
%    0.3044
%    0.3235
%   -0.0752
%   -0.0601

%Starting values for b are not stellar. As seen from these correlations.
%Try to refine the b estimates next
corr(InfDS0.trueb,InfDS.b)
cov(InfDS.b)
cov(InfDS0.trueb)

%Filtered estimates for state variables don't look too bad even though we
%are just using these initial estimates.
if (plot_flag == 1)
        figure(2)
        clf;
        plot(InfDS.timeDiscrete{1},InfDS0.trueX{1,1},'k-','Linewidth',2);hold on;
        plot(InfDS.timeDiscrete{1},reshape(Xh_ekf(1,1,:),1,length(InfDS.timeDiscrete{i})),'r-.','Linewidth',1);
        for i=2:30
            if max(abs(reshape(Xh_ekf(1,i,:),1,InfDS.allT(i)))) < 100
                plot(InfDS.timeDiscrete{i},InfDS0.trueX{i,1},'k-','Linewidth',2);
                plot(InfDS.timeDiscrete{i},reshape(Xh_ekf(1,i,:),1,InfDS.allT(i)),'r-.','Linewidth',1);
            end
        end
        hold off;
        xlabel('\fontsize{14} Time');
        ylabel('\fontsize{14} EKF estimates');
        title('\fontsize{14} After 10 iterations: true (-) and estimated (r-.)');
end

%save After2a.mat % In case anyone wants to save the intermediate results

disp '(2b) Get better starting values for the random effects in b';

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Get better starting values for the the random effects in InfDS.b
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
upperb = [5,5,5]; lowerb= [-5,-5,-5]; %Upper and lower boundary values for b - system gets explosive
for q=1:InfDS.Nb
   InfDS.b(InfDS.b(:,q) < lowerb(q) | InfDS.b(:,q) > upperb(q),q) = 0; 
end

startb = InfDS.b;
Q2 = cov(InfDS.b);
Q = [1, Q2(2,2), Q2(3,3), Q2(2,3)];
InfDS.start(9:12) = [log(Q(1)), log(Q(2)),  log(Q(3) - Q(4)^2/Q(2)), Q(4)/Q(2) ];
InfDS.par(9:12) = [log(Q(1)), log(Q(2)),  log(Q(3) - Q(4)^2/Q(2)), Q(4)/Q(2) ];


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Set up parameters for draw b
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Resetting a bunch of control parameters to reflect the model structure
%with only state variables in x (no dynamic parameters).
InfDS.Nx=2;
InfDS.G = eye(InfDS.Nx);
InfDS.Nbeta = 5;
InfDS.Ntheta = 3;
InfDS.Neta=0;
InfDS.Npar0 = InfDS.Nmu + InfDS.NLambda + InfDS.Ny + InfDS.Nbpar;
InfDS.alp=1;
InfDS.odefn    = @dynfunIC;      % Function for ODE solver 
InfDS.dfdx = @dfdxFreeIC;
InfDS.dfdxdp = @dfdxdpFreeIC;
InfDS.dfdpdx = @dfdpdxFreeIC;
InfDS.dfdp = @dfdparFreeIC;
InfDS.dfdp2 = @dfdpar2FreeIC;
InfDS.dfdx2 = @dfdx2FreeIC;

%Now stick in the parameter estimates from running the EKF previously to
%InfDS.par
InfDS.par = [reshape(temp0(1:5), InfDS.Nbeta, 1);reshape([InfDS.startpars],InfDS.Npar0,1)];

stop = 0;
InfDS.N = 1; % Lu modified, 04-12-13, 5;
STARTGIB=0;
yesMean = 0;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
InfDS.y0 = repmat([0 0],InfDS.Nsubj,1);
InfDS.sy = zeros(length(InfDS.par),1);
InfDS.ES = zeros(length(InfDS.par),length(InfDS.par));
InfDS.EI = zeros(length(InfDS.par),length(InfDS.par));
sgnTH = zeros(length(InfDS.par),InfDS.maxIterStage1);
InfDS.Iy = zeros(length(InfDS.par),length(InfDS.par));
InfDS.thetatild = zeros(length(InfDS.par),1);
InfDS.sytild = zeros(length(InfDS.par),1); 
InfDS.EStild = zeros(length(InfDS.par),length(InfDS.par));
InfDS.EItild = zeros(length(InfDS.par),length(InfDS.par));
InfDS.Iytild = zeros(length(InfDS.par),length(InfDS.par));


InfDS.lowBound = NaN*ones(length(InfDS.par),1);
InfDS.upBound = NaN*ones(length(InfDS.par),1);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%self change, with caution and ask
InfDS.lowBound(9:10) = 10e-8;
InfDS.upBound(9:10) = 10;
InfDS.lowBound(11:16) = log(10e-8);
InfDS.upBound(11:16) = log(20);

InfDS.IT = 2; %Time point after initial condition
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tmp = zeros(InfDS.Nsubj,length(InfDS.tspan));
for i= 1:InfDS.Nsubj
for t = 1:length(InfDS.tspan)
	if min(abs(InfDS.tobs{i}-t)) == 0 
		tmp(i,t) = 1;
	end
end
end
InfDS.tidx = tmp; %logical(tmp);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[InfDS] = setParsFreeICwb(InfDS);
InfDS = getXtildIC3(InfDS,0,freeIC); %Get updated Xtilde
 InfDS.b = startb;
meanb = startb;
useMultN = 1; freeIC = 1; InfDS.ICb = 2:3;
InfDS.bAdaptParams = [.5,2.5,.5];%[.5,2,.5]; % Lu 04-12-13  [.1,10,.5,.5,2.5,.5]; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[InfDS] = setParsFreeICwb(InfDS);
InfDS = getXtildIC3(InfDS,1,freeIC); %Get updated Xtilde
[InfDS] = PropSigb(InfDS); %covariance of proposal distribution of b
[tpOld] = ekfContinuous10(InfDS); %get density of full conditional distribution of b 

InfDS.scaleb = 100; 
InfDS.bacc = zeros(InfDS.Nsubj,1);	
InfDS.setAccept = .7; ibmax=50;
%First do a 1-d search
isBlock1Only = 0; setScaleb = 1;

for ib = 1:ibmax
    if ib == 2
        isBlock1Only = 1;
        %setScaleb = 1;
    end

    if ib == ibmax-20
        InfDS.scaleb = 1;
        InfDS.bAdaptParams = [.1,10,.5];
        InfDS.setAccept = .5;       
        isBlock1Only = 0; 
        setScaleb = 1;
    end
    
    if ib > ibmax-10
        InfDS.scaleb = 1;
        InfDS.bAdaptParams = [.1,2.5,.5];
       setScaleb = 0; 
        Q2 = cov(InfDS.b);
        Q = [Q2(1,1), Q2(2,2), Q2(3,3), Q2(2,3)];
        InfDS.par(14:17) = [log(Q(1)), log(Q(2)),  log(Q(3) - Q(4)^2/Q(2)), Q(4)/Q(2) ];
        [InfDS] = setParsFreeICwb(InfDS);
    end

[InfDS,bAccept,meanb,tpOld] = drawbGeneral6_opt3(InfDS,meanb,yesMean,upperb,lowerb,useMultN,tpOld,freeIC,isBlock1Only,setScaleb);
%%:-) Comment out during Monte Carlo simulations
fprintf('\nGIB = %5d\n',ib);
fprintf('\nCurrent b acceptance rate = %6f\n',bAccept);
fprintf('\nRange of trueb 1 = %6f, %6f',min(InfDS0.trueb(:,1)),max(InfDS0.trueb(:,1)));
fprintf('\nRange of bhat 1 = %6f, %6f',min(InfDS.b(:,1)),max(InfDS.b(:,1)));
fprintf('\nCorr = '); corr(InfDS.b(:,1),InfDS0.trueb(:,1))
fprintf('\nScaleb = ');InfDS.scaleb
if size(InfDS0.trueb,2) > 1
fprintf('\nRange of trueb 2 = %6f, %6f',min(InfDS0.trueb(:,2)),max(InfDS0.trueb(:,2)));
fprintf('\nRange of bhat 2 = %6f, %6f',min(InfDS.b(:,2)),max(InfDS.b(:,2)));
fprintf('\nRange of trueb 3 = %6f, %6f',min(InfDS0.trueb(:,3)),max(InfDS0.trueb(:,3)));
fprintf('\nRange of bhat 3 = %6f, %6f',min(InfDS.b(:,3)),max(InfDS.b(:,3)));
fprintf('\nCorr = '); corr(InfDS.b(:,2:3),InfDS0.trueb(:,2:3))
end

end %End of loop through ibmax

%save After2b.mat % Intermediate results

%load After2b.mat %If you don't want to restart from scratch. But beware to
%remove this line to avoid reading in the same estimation results over and
%over
disp '(2c) Start EKF again: Get better starting values for the dynamic parameters';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (2c) Run EKF again to jointly estimate state variables and beta again to
% improve starting values
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
InfDS.Nx = 7;%Number of latent variables
InfDS.Nbeta = 0; %Size of beta (fixed effects for dynamic parameters);
InfDS.Nbpar = 4;
InfDS.N = 1;

InfDS.odefn    = @dynfunParIC;      % Function for ODE solver (exact)RHS Functions
InfDS.Jdyn  = @dfdxParIC; %For ekf
InfDS.Nbetax = InfDS.Nx-InfDS.NxState; %Fixed effects parameters that are in state vector

    InfDS.P0 = 1e-7*eye(InfDS.Nx);
    InfDS.P0(3,3) = 1;
    InfDS.P0(4,4) = .1;
    InfDS.P0(5,5) = .1;
    InfDS.P0(6:7,6:7) = InfDS.Sigmab(2:3,2:3);
    InfDS.G = eye(InfDS.Nx);
    Oldpar = InfDS.par;
    InfDS.par = InfDS.par(6:17);

    InfDS.sy = zeros(length(InfDS.par),1);
    InfDS.ES = zeros(length(InfDS.par),length(InfDS.par));
    InfDS.EI = zeros(length(InfDS.par),length(InfDS.par));
    InfDS.Iytild = zeros(length(InfDS.par),length(InfDS.par));
    InfDS.Iy = zeros(length(InfDS.par),length(InfDS.par));
    InfDS.lowBound = NaN*ones(length(InfDS.par),1);
    InfDS.upBound = NaN*ones(length(InfDS.par),1);
    InfDS.lowBound(4:5) = 10e-8;
    InfDS.upBound(4:5) = 10;
    InfDS.lowBound(6:11) = log(10e-7);
    InfDS.upBound(6:11) = log(10);
    InfDS.N = 1; freeIC = 0;%FreeIC always = 0 in this EKF part

    
    InfDS.y0 = repmat([0 0 Oldpar(1:5)'],InfDS.Nsubj,1); 

     
    gmm0=1;sgnTH0 = zeros(length(InfDS.par),1);stage=0; 
    for k=1:5
    [InfDS] = setParsEKFFreeICwb(InfDS);
    [InfDS,~,Xh_ekf] = ekfContinuous4(InfDS,-2);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    [score,infoMat] =  getScoreInfoY_tobs_opt(InfDS,stage,k,freeIC);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    [InfDS, stop0, ~, gmm0] = saem(InfDS,gmm0,0,sgnTH0, score,score*score',infoMat,{});
    InfDS = getXtildIC3(InfDS,0,freeIC); %Get updated Xtilde

     %x1 = mean(reshape(Xh_ekf(InfDS.NxState+1:InfDS.Nx,:,(InfDS.Nx+1):InfDS.totalT),InfDS.Nbetax,(InfDS.totalT-InfDS.Nx)*InfDS.Nsubj),2);
     x1 = mean(reshape(Xh_ekf(InfDS.NxState+1:InfDS.Nx,:,InfDS.totalT),InfDS.Nbetax,(1)*InfDS.Nsubj),2);
     InfDS.y0 = repmat([0,0 x1'],InfDS.Nsubj,1);
        
     for ss=1:InfDS.Nsubj
            InfDS.b(ss,2) = Xh_ekf(6,ss,InfDS.allT(ss)) - x1(4);
            InfDS.b(ss,3) = Xh_ekf(7,ss,InfDS.allT(ss)) - x1(5); 
     end
     
    end   
   
%%:-) Comment out during Monte Carlo simulations
fprintf('\n In 2(c) Running EKF again for better starting values');
fprintf('\nOld beta from (2a) = '); InfDS.par(1:5)
fprintf('\nOld beta after(2c) = '); x1;
    
fprintf('\nUpdated b corr = '); corr(InfDS0.trueb(:,1:size(InfDS0.trueb,2)),InfDS.b)
fprintf('\nUpdated b var-cov = '); 
cov(InfDS.b)
cov(InfDS0.trueb)    

%Use empirical var-cov estimates for InfDS.b as starting values for
%parameters in Sigma_b
Q2 = cov(InfDS.b);
Q = [Q2(1,1), Q2(2,2), Q2(3,3), Q2(2,3)];
InfDS.par(9:12) = [log(Q(1)), log(Q(2)),  log(Q(3) - Q(4)^2/Q(2)), Q(4)/Q(2) ];
    
%save After2c.mat

% End of (2c) EKF again
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp '(3) Start SAEM';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (3) Set up parameters for and run SAEM
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%If restarting part (3) with intermediate results loaded
%load After2c.mat
%addpath('../UpdatedFunctions/');
%addpath('./dxFileswithGetXTildFreeIC/');

InfDS.par = [x1; InfDS.par];

InfDS.Nx=2;
InfDS.G = eye(InfDS.Nx);
InfDS.Nbeta = 5;
InfDS.Ntheta = 3;
InfDS.alp=1;
InfDS.odefn    = @dynfunIC;      % Function for ODE solver 
InfDS.dfdx = @dfdxFreeIC;
InfDS.dfdxdp = @dfdxdpFreeIC;
InfDS.dfdpdx = @dfdpdxFreeIC;
InfDS.dfdp = @dfdparFreeIC;
InfDS.dfdp2 = @dfdpar2FreeIC;
InfDS.dfdx2 = @dfdx2FreeIC;
InfDS.y0 = repmat([0 0],InfDS.Nsubj,1);
InfDS.sy = zeros(length(InfDS.par),1);
InfDS.ES = zeros(length(InfDS.par),length(InfDS.par));
InfDS.EI = zeros(length(InfDS.par),length(InfDS.par));
sgnTH = zeros(length(InfDS.par),InfDS.maxIterStage1);
InfDS.Iy = zeros(length(InfDS.par),length(InfDS.par));
InfDS.thetatild = zeros(length(InfDS.par),1);
InfDS.sytild = zeros(length(InfDS.par),1); 
InfDS.EStild = zeros(length(InfDS.par),length(InfDS.par));
InfDS.EItild = zeros(length(InfDS.par),length(InfDS.par));
InfDS.Iytild = zeros(length(InfDS.par),length(InfDS.par));


InfDS.lowBound = NaN*ones(length(InfDS.par),1);
InfDS.upBound = NaN*ones(length(InfDS.par),1);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%self change, with caution and ask
InfDS.lowBound(9:10) = 10e-8;
InfDS.lowBound(11:16) = log(10e-8);
InfDS.lowBound(17) = -2; %After transformation, covariance between -5 and 5

InfDS.upBound(9:10) = 10;
InfDS.upBound(11:16) = log(10);
InfDS.upBound(11:16) = 2;


k=1; stage=1; gmm=1;
MAXGIB=1;
InfDS.errtrol1 = 1.5; %Stage 1 error tolerance

InfDS.errtrol = 1.5; %Stage 2 error tolerance


setScaleb = 0;
InfDS.bAdaptParams = [.5,2.5,.5];
ssmin = 100; noIncrease = 0;
freeIC = 1;
InfDS.scaleb = 1; %Used in drawbGenera6_opt3.m to determine whether to apply scaling constant on drawb.
InfDS.KKO = 20; %Used in SAEM. Only starts to evaluate whether to transition to stage 2 after KKO iterations.
%tic;
while (k<InfDS.MAXITER && stop==0),
[InfDS] = setParsFreeICwb(InfDS);%qqqq
    

if stage==2 && switchFlag==0
yesMean= 1;
switchFlag = 1; 
meanb = zeros(InfDS.Nsubj,InfDS.Nb); %Keeps b estimates averaged across Gibbs runs
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%       

    
if stage==1 && bAccept < .1 %Not looking too good. Pump up no. of chains
            useMultN = 1; 
elseif stage==1 && k == 3
            useMultN = 1; MAXGIB = 30;
elseif stage==1 && k <= 4 && k~= 3
            useMultN = 0; MAXGIB = 1;
else 
    useMultN = 0; MAXGIB = 10;
end

    mscore = zeros(length(InfDS.par),1);
    mscore2 = zeros(length(InfDS.par),length(InfDS.par));
    minfoMat = zeros(length(InfDS.par),length(InfDS.par));  
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Gibbs sampler
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
	%Since beta changes, get Xtild and InfDS.InfDS.dxstardthetafAll and InfDS.dxstardthetafAll2
	InfDS = getXtildIC3(InfDS,1,freeIC); %Get updated Xtilde
	[InfDS] = PropSigb(InfDS); %covariance of proposal distribution of b
	[tpOld] = ekfContinuous10(InfDS); %get density of full conditional distribution of b 
	
	InfDS.bacc = zeros(InfDS.Nsubj,1);	
    for GIB=1:MAXGIB,
        %fprintf('\nGIB = %5d = \n',GIB);
        %tic;
        if (stage==2),
            yesMean = 1;
            STARTGIB = STARTGIB+1;
        end

        if (k >= 4)
        [InfDS,bAccept,meanb,tpOld] = drawbGeneral6_opt3(InfDS,meanb,yesMean,upperb,lowerb,useMultN,tpOld,freeIC,isBlock1Only,setScaleb);
 
        
        %fprintf('\nCurrent b acceptance rate = %6f\n',bAccept);
        %fprintf('\nRange of InfDS0.trueb = %6f, %6f',min(InfDS0.trueb),max(InfDS0.trueb));
        %fprintf('\nRange of bhat = %6f, %6f',min(InfDS.b),max(InfDS.b));
        %corr(InfDS.b,InfDS0.trueb)
        end
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %toc
        
        %Calculate summary statistics for E-step of scoring algorithm
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        [score,infoMat] =  getScoreInfoY_tobs_opt(InfDS,stage,k,freeIC);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %disp('mscore');
        %disp(mscore);
        %disp('mscore2');
        %disp(mscore2);
        %disp('minfoMat');
        %disp(minfoMat);
        
        mscore = mscore+(1/MAXGIB)*score;
        mscore2 = mscore2+(1/MAXGIB)*(score*score');
        minfoMat = minfoMat+(1/MAXGIB)*infoMat;
        
    end%end of Gibbs sampler loop
    
    Covscore = mscore2 - mscore*mscore';
    [InfDS, stop, stage, gmm,ss,sgnTH,redFlag,ssmin,noIncrease,convFlag] = saem(InfDS,gmm,stage,sgnTH,mscore,mscore2,minfoMat,Covscore,ssmin,noIncrease);
    
    
    fprintf('\nStage = %5d, iteration = %5d\n',stage,k);
    fprintf('\nCurrent b acceptance rate = %6f\n',bAccept);
    fprintf('\nRange of InfDS0.trueb = %6f, %6f',min(InfDS0.trueb),max(InfDS0.trueb));
    fprintf('\nRange of bhat = %6f, %6f',min(InfDS.b),max(InfDS.b));
    corr(InfDS.b(:,1:size(InfDS0.trueb,2)),InfDS0.trueb)

    InfDS.par([1:10])
    exp(InfDS.par([11:13]))'
    InfDS.Sigmab
    disp 'Averaging:'
    InfDS.thetatild([1:10])
    exp(InfDS.thetatild([11:13]))'
    D = diag(exp(InfDS.thetatild(14:16)));
    L = [1 0 0;
    0 1 0;
    0 InfDS.thetatild(17) 1];
    QQ = L*D*L';

    
    disp(k);

    %%%%%%%%%%
    k = k+1;
    
end %end of scoring iteration loop
ttt = toc;
fprintf('\nFinal iteration = %5d, ss = %5f, time = %5f \n', k,ss,ttt);


meanb = meanb./STARTGIB;
InfDS.par = InfDS.thetatild;

save after3.mat;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (4) Write out results
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp '(4) Wrap up estimation and write out results';


%fprintf('\nRange of InfDS0.trueb = %6f, %6f',min(InfDS0.trueb),max(InfDS0.trueb));
%fprintf('\nRange of final bhat = %6f, %6f',min(meanb),max(meanb));
%corr(meanb,InfDS0.trueb)

%If using transformation functions
dgdpar = eye(size(InfDS.par,1));
dgdpar(11:13,11:13) = diag(exp(InfDS.par(11:13)));

%%%%%
%Sigmab looks like this currently with the LDL' decomposition:
%[      exp(par1),          0,                   0]
%[      0,          exp(par2),      par4*exp(par2)]
%[      0,          par4*exp(par2), exp(par2)*par4^2 + exp(par3)]
%%
%Columns -- par, rows --transformation function
dgdpar(14,14) = exp(InfDS.par(14));
dgdpar(15,15) = exp(InfDS.par(15));
dgdpar(16,15) = InfDS.par(17)*exp(InfDS.par(15));%diff(f1,par2)
dgdpar(16,17) = exp(InfDS.par(15));%diff(f1,par4)
dgdpar(17,15) = InfDS.par(17)^2*exp(InfDS.par(15));%diff(f2,par2)
dgdpar(17,16) = exp(InfDS.par(16));%diff(f2,par3)
dgdpar(17,17) = 2*InfDS.par(17)*exp(InfDS.par(15));%diff(f2,par4)


InfDS.SE = sqrt(diag(dgdpar/InfDS.Iytild*dgdpar'));
InfDS.par(11:13) = exp(InfDS.par(11:13));
InfDS.par(14:17) = [QQ(1,1), QQ(2,2), QQ(3,3), QQ(2,3)];
InfDS.lastb = InfDS.b; InfDS.meanb = meanb;
InfDS.convFlag = convFlag;
InfDS.ss = ss;


fitInit = 1; %Fit models with freely estimated IC.
dlmwrite(filenamePar,[trueInit fitInit batch kk reshape(InfDS.par,1,length(InfDS.par))],'-append');
dlmwrite(filenameSE,[trueInit fitInit batch kk reshape(InfDS.SE,1,length(InfDS.par))],'-append');
dlmwrite(filenameconv,[trueInit fitInit batch kk InfDS.convFlag InfDS.ss],'-append');
dlmwrite(filenamebhat,[trueInit fitInit batch kk reshape(InfDS.lastb,1,InfDS.Nsubj*InfDS.Nb)],'-append'); %Reshape by column
dlmwrite(filenamebhat2,[trueInit fitInit batch kk reshape(InfDS.meanb,1,InfDS.Nsubj*InfDS.Nb)],'-append');%Reshape by column


%end %for loop through Monte Carlo runs