function [InfDS] = getXtildIC3(InfDS, getDxFlag, freeIC)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% -getDxFlag: 0 = just getting xtild, not getting other
%                 elements needed for the computation of s_Z and I_Z; 
%             1 = want other elements as well.
% -freeUC:0 = Not freely estimating IC; 1 = freely estimating IC
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Get Xtilde and dXtild/dtheta_f and dvec(dXtild/dtheta_f)/dtheta_f^T
InfDS.Xtild = zeros(InfDS.Nx,InfDS.Nsubj,InfDS.totalT);
if getDxFlag ==1
    InfDS.dXstarAll = cell(InfDS.Nsubj);
    InfDS.dXstarAll2 = cell(InfDS.Nsubj);
    InfDS.dXtildthetafAll = cell(InfDS.Nsubj);
    InfDS.dXtildthetafAll2 = cell(InfDS.Nsubj);
end


tspan = InfDS.tspan;
fullX = zeros(InfDS.Nx,InfDS.Nsubj,length(tspan));

T = length(tspan);%InfDS.lens(i);
xk1 = zeros(InfDS.Nx,InfDS.Nsubj,T);
xk2 = zeros(InfDS.Nx,InfDS.Nsubj,T);
tindex = InfDS.tspan;


dt = repmat(InfDS.delt,T,1);
Nsubj = InfDS.Nsubj;
tcount = ones(Nsubj,1);

for i= 1:InfDS.Nsubj

    if getDxFlag ==1
        d2Xtild = zeros(InfDS.Ntheta*InfDS.Nx,InfDS.Ntheta); %Running sum of second derivatives
        InfDS.dXstarAll{i} = zeros(InfDS.Ntheta,InfDS.Nx*InfDS.allT(i));
        InfDS.dXstarAll2{i} = zeros(InfDS.Ntheta*InfDS.Nx*InfDS.allT(i),InfDS.Ntheta);
        InfDS.dXtildthetafAll{i} = zeros(InfDS.Ntheta,InfDS.Nx*InfDS.allT(i));
        InfDS.dXtildthetafAll2{i} = zeros(InfDS.Ntheta*InfDS.Nx*InfDS.allT(i),InfDS.Ntheta);
    end
end


if (freeIC==1)
    XtildPrev = feval(InfDS.odefn, InfDS.y0',0,InfDS,[],0,1);
    dXtildPrev0 = [0,0;
                   1,0;
                   0,1];
    d2XtildPrev0 = zeros(InfDS.Nx*InfDS.Ntheta,InfDS.Ntheta);
else
    XtildPrev = feval(InfDS.odefn, InfDS.y0',0,InfDS,[],0,1); %InfDS.y0(:,1:InfDS.Nx)';
    dXtildPrev0 = zeros(InfDS.Ntheta,InfDS.Nx); %dXtild = dXtildPrev;
    d2XtildPrev0 = zeros(InfDS.Ntheta*InfDS.Nx,InfDS.Ntheta); %d2Xtild = d2XtildPrev;
end


if getDxFlag ==1
    dXtildPrev = reshape(repmat(dXtildPrev0,1,Nsubj),InfDS.Ntheta,InfDS.Nx,Nsubj);
    dXtild = dXtildPrev;
    d2XtildPrev = reshape(repmat(d2XtildPrev0,1,Nsubj),InfDS.Ntheta*InfDS.Nx,InfDS.Ntheta,Nsubj);
    d2Xtild = d2XtildPrev;
    dXstar_t = dXtild;
    d2Xstar_t = d2Xtild;
end

InfDS.Xtild(:,:,1) = XtildPrev;
fullX(:,:,1) = XtildPrev;
if getDxFlag ==1    
    for i = 1:InfDS.Nsubj
        InfDS.dXtildthetafAll{i}(:,(1:InfDS.Nx)) = dXtildPrev(:,:,i);
        InfDS.dXtildthetafAll2{i}(1:(InfDS.Ntheta*InfDS.Nx),:) = d2XtildPrev(:,:,i);
    end
end

for t=2:T %Do interpolation at small, equal intervals, as opposed to at observed intervals to avoid numerical problems
    
    %ODE solver
    k1 = feval(InfDS.odefn, XtildPrev,0,InfDS,[],tindex(t),0); % ~ i
    k21= XtildPrev+dt(t).*k1;
    k2 = feval(InfDS.odefn, k21,0,InfDS,[] ,tindex(t)+dt(t),0); % ~ i
    Xtild_t = XtildPrev+dt(t)*(k1+k2)/2;
    xk1(:,:,t) = k1;
    xk2(:,:,t) = k2;
    
    
    if getDxFlag==1 && t > 2
        dk1dtheta = feval(InfDS.dfdp,XtildPrev,InfDS,[],tindex(t));
        dk2dtheta = feval(InfDS.dfdp,k21,InfDS,[],tindex(t)+dt(t));
        
        %Note that dfdx has x in rows, f in columns, unlike the Jacobian function
        %used in ekf, specified in InfDS.Jdyn
        dfdxATXtildprev = feval(InfDS.dfdx,XtildPrev,InfDS,[],tindex(t));
        dk1 = zeros(InfDS.Ntheta,InfDS.Nx,Nsubj);
        for i = 1:Nsubj
            dk1(:,:,i) = dXtildPrev(:,:,i)*dfdxATXtildprev(:,:,i) +dk1dtheta(:,:,i);
        end
        dk21 = dXtildPrev + dt(t)*dk1; %dk21/dthetaf
        
        dfdxATk21 = feval(InfDS.dfdx,k21,InfDS,[],tindex(t));%df/dx evaluated at k21
        dk2 = zeros(InfDS.Ntheta,InfDS.Nx,Nsubj);
        for i = 1:Nsubj
            dk2(:,:,i) = dk21(:,:,i)*dfdxATk21(:,:,i) +dk2dtheta(:,:,i);
        end
        
        dXstar_t = dt(t)/2*(dk1 + dk2);
        dXtild = dXtildPrev + dXstar_t;
        
        dvecdfdXtilddtheta = feval(InfDS.dfdxdp,XtildPrev,InfDS,[],tindex(t));%dfdxdp
        dvecdfdXtilddXtild = feval(InfDS.dfdx2,XtildPrev,InfDS,[],tindex(t)); %dfdx2
        dveck1dthetadXtild = feval(InfDS.dfdpdx,XtildPrev,InfDS,[],tindex(t));%dfdpdx
        dveck1dtheta = feval(InfDS.dfdp2,XtildPrev,InfDS,[],tindex(t)); %dfdp2 with XtildPrev
        
        d2vecdk1= zeros(InfDS.Nx*InfDS.Ntheta,InfDS.Ntheta,Nsubj);
        first= zeros(InfDS.Nx*InfDS.Ntheta,InfDS.Ntheta,Nsubj);
        for i = 1:Nsubj
            d2vecdk1(:,:,i) = ...
                (kron(dfdxATXtildprev(:,:,i),eye(InfDS.Ntheta))'* d2XtildPrev(:,:,i) + ...%2a
                kron(eye(InfDS.Nx),dXtildPrev(:,:,i))*(dvecdfdXtilddtheta(:,:,i) + ...
                dvecdfdXtilddXtild(:,:,i)*dXtildPrev(:,:,i)') + ...%2a
                dveck1dtheta(:,:,i) + dveck1dthetadXtild(:,:,i)*dXtildPrev(:,:,i)'); %part 2b
            first(:,:,i) = (d2XtildPrev(:,:,i) + dt(t)*d2vecdk1(:,:,i));
        end
        
        dvecdfdxATk21dtheta  = feval(InfDS.dfdxdp,k21,InfDS,[],tindex(t)+dt(t));%dfdxdp
        dvecdfdxATk21dk21 = feval(InfDS.dfdx2,k21,InfDS,[],tindex(t)+dt(t)); %dfdx2
        second = zeros(InfDS.Nx^2,InfDS.Ntheta,Nsubj);
        for i = 1:Nsubj
            second(:,:,i) = dvecdfdxATk21dtheta(:,:,i) + dvecdfdxATk21dk21(:,:,i)*dk21(:,:,i)';%Second term, dvecfxdxatk21_2
        end
        dveck2dtheta = feval(InfDS.dfdp2,k21,InfDS,[],tindex(t)+dt(t)); %dfdp2 with k21
        dveck2dthetadxATk21 = feval(InfDS.dfdpdx,k21,InfDS,[],tindex(t)+dt(t));%dfdpdx
        third = zeros(InfDS.Nx*InfDS.Ntheta,InfDS.Ntheta,Nsubj);
        d2vecdk2 = zeros(InfDS.Nx*InfDS.Ntheta,InfDS.Ntheta,Nsubj);
        for i = 1:Nsubj
            third(:,:,i) = dveck2dtheta(:,:,i)  + dveck2dthetadxATk21(:,:,i)*dk21(:,:,i)'; %third term, dvecd2k2dtheta_2
            d2vecdk2(:,:,i) = ...
            kron(dfdxATk21(:,:,i),eye(InfDS.Ntheta))'* first(:,:,i) + ...
            kron(eye(InfDS.Nx),dk21(:,:,i))*(second(:,:,i)) +...
            third(:,:,i);
        end
        
        d2Xstar_t = dt(t)/2*(d2vecdk1 + d2vecdk2);
        
        d2Xtild = d2XtildPrev + d2Xstar_t;
        dXtildPrev = dXtild;
        d2XtildPrev = d2Xtild;
    end %end of getDXFlag
    
    XtildPrev = Xtild_t;

    
    for i= 1:InfDS.Nsubj
        %currentt = indext(InfDS.timeDiscrete{i}==tspan(t));
        %This needs to be fixed. No need to sum again in getScoreInfo
        indext = 1:InfDS.allT(i);
        currentt = indext(InfDS.timeDiscrete{i}==tspan(t));

        fullX(:,i,t) =Xtild_t(:,i);

        if ~isempty(currentt)
        %if min(abs(InfDS.tobs{i}-t))==0 && ~isempty(currentt)
        %if InfDS.tidx(i,t)==1
            
            %currentt = tcount(i);
            
            if getDxFlag ==1
                InfDS.dXtildthetafAll{i}(:,(1+(currentt-1)*InfDS.Nx):(currentt*InfDS.Nx)) = dXtild(:,:,i);
                InfDS.dXtildthetafAll2{i}((1+(currentt-1)*InfDS.Ntheta*InfDS.Nx):(currentt*InfDS.Ntheta*InfDS.Nx),:) = d2Xtild(:,:,i);
                InfDS.dXstarAll{i}(:,(1+(currentt-1)*InfDS.Nx):(currentt*InfDS.Nx)) = dXstar_t(:,:,i);
                InfDS.dXstarAll2{i}((1+(currentt-1)*InfDS.Ntheta*InfDS.Nx):(currentt*InfDS.Ntheta*InfDS.Nx),:) = d2Xstar_t(:,:,i);
            end
            tcount(i) = tcount(i) + 1;
        end
    end %end of nsubj loop
    
end %t loop

for i=1:InfDS.Nsubj
   InfDS.Xtild(:,i,1:InfDS.allT(i)) = fullX(:,i,InfDS.tobs{i}); 
end

end