function r = dynfunParIC(xin,isCell,InfDS,i,t,isStart)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% The ODE functions for the Van der Pol oscillator, with dynamic parameters
% in theta_fi represented as part of the state vector.
% ----
% Note that t is not used but kept because some of the estimation functions
% are reused from time to time in various applications and some of them
% do have t in the ODE functions.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

y = xin;
if (isCell==1),
r = cell(InfDS.Nx,1);
else 
r = y;
end;


omega = InfDS.omega;


%%%%%%%%%%%%%%%%%%%%
%Compute theta_fi
%%%%%%%%%%%%%%%%%%%%
pari = zeros(InfDS.Ntheta, size(y,2));
if ~isempty(i)
Hi = InfDS.H((1+(i-1)*InfDS.Ntheta):(i*InfDS.Ntheta),:);
pari(1) = y(3)*Hi(1,1)+y(4)*Hi(1,2)+y(5)*Hi(1,3);
pari(2) = y(6)*Hi(2,4);  
pari(3) = y(7)*Hi(3,5);
else
rowNum = 1:size(InfDS.H,1);
aa1 = kron(InfDS.H(mod(rowNum,InfDS.Ntheta)==1,:)',ones(1,InfDS.N));
aa2 = kron(InfDS.H(mod(rowNum,InfDS.Ntheta)==2,:)',ones(1,InfDS.N));
aa3 = kron(InfDS.H(mod(rowNum,InfDS.Ntheta)==0,:)',ones(1,InfDS.N));

aa1(1,:) = y(3,:).*aa1(1,:);
aa1(2,:) = y(4,:).*aa1(2,:);
aa1(3,:) = y(5,:).*aa1(3,:);
pari(1,:) = sum(aa1(1:3,:),1); 
pari(2,:) = y(6,:).*aa2(4,:);
pari(3,:) = y(7,:).*aa3(5,:);   
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get b to be used in ODE functions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
b = zeros(size(pari));
if (InfDS.Nb > 0)
if ~isempty(i) %Only for one subject, but may have t > 1
b(1:InfDS.Ntheta,:) = kron(ones(1,size(y,2)),InfDS.Z*reshape(InfDS.b(i,:),InfDS.Nb,1));    
elseif mod(size(y,2),InfDS.Nsubj)==0 %for more than one subject but one t
b = InfDS.Z*InfDS.b';    
else %for more than one subject and more than one t
   bb = InfDS.Z*InfDS.b';    
   bb = kron(bb,ones(1,max(InfDS.allT)));
   b = bb(:,InfDS.Tfilter==1);        
end
end

%Add together the fixed and random effects to get theta_fi = pari
pari = pari+b;

if (isCell==1),
  
if isStart==1
    r{1} = pari(2,:);
    r{2} = pari(3,:);
    r{3} = y{3};
    r{4} = y{4};
    r{5} = y{5};
    r{6} = pari(2,:);
    r{7} = pari(3,:);
else
r{1} = cell2mat(y{2})';
r{2} = -omega*cell2mat(y{1})' +pari(1,:).*(1-cell2mat(y{1})'.^2).*cell2mat(y{2})';
r{3} = 0;
r{4} = 0;
r{5} = 0;
r{6} = 0;
r{7} = 0;  
end
else %if not isCell==1


if isStart==1
    r(1,:) = pari(2,:);
    r(2,:) = pari(3,:);
    r(3,:) = y(3,:);
    r(4,:) = y(4,:);  
    r(5,:) = y(5,:);
    r(6,:) = pari(2,:);
    r(7,:) = pari(3,:);
else
r(3,:) = zeros(1,size(y,2));
r(4,:) = zeros(1,size(y,2));
r(5,:) = zeros(1,size(y,2));
r(6,:) = zeros(1,size(y,2));
r(7,:) = zeros(1,size(y,2));
r(1,:) = y(2,:);
r(2,:) = -omega*y(1,:) +pari(1,:).*(1-y(1,:).^2).*y(2,:);    
end
end    

end

