function [InfDS,minDelt,maxDelt] = genDataFreeICwb(plot_flag,trueInit,filenametrueb,filenameY,batch,kk)
disp 'Generate data: free IC with b';
InfDS = {};

InfDS.Nx = 7;%Number of latent variables
InfDS.Ny = 3;%Number of manifest variables
InfDS.Nbeta = 0; %Size of beta (fixed effects parameters for elements in theta_i); Set to 0 here
                 %because data generation was done by sticking the fixed
                 %effects parameters into the latent state vector. Later
                 %during model fitting, this will be set to 5 corresponding
                 %to beta = [\zeta_0 \zeta_1 \zeta_2 \mu_{x1} and
                 %\mu_{x2}]'
InfDS.Ntheta = 3; %Size of theta_fi: dynamic parameters with subject-specific parameters (fixed and random effects)
InfDS.Nb = 3;%Number of random effects (b_zeta, b_x1, b_x2)
InfDS.Nmu = 3; %Number of intercept parameters (mu_1, mu_2, and mu_3)
InfDS.NLambda = 2; %Number of parameters in factor loading matrix
InfDS.Nbpar = 4; %Number of parameters in random effects variance-covariance matrix (Eq 15)
InfDS.Nu = 2; %Number of covariates predicting the person-specific theta_fi
%InfDS.N = 1; %Not used any more. At one point was doing multiple chains of
% MCMC samples concurrently.

InfDS.NxState=2; %Number of latent variables (x_1, x_2)
InfDS.odefn    = @dynfunParIC;      % File name for ODE functions of the Van der Pol - parameters in state vector
InfDS.Jdyn  = @dfdxParIC; %File name for Jacobian matrix of state variables. Not used in data gen but needed for ekf later.
InfDS.yfun = @VDPMeas; %Measurement functions
InfDS.Nbetax = InfDS.Nx-InfDS.NxState; %Number of fixed effects parameters that are in state vector
InfDS.Npar0 = InfDS.Nmu + InfDS.NLambda + InfDS.Ny + InfDS.Nbpar; %Total number of parameters

%Set up true values of parameters
InfDS.isInfo = 1;
InfDS.totalT = 300;
InfDS.Nsubj = 200;
InfDS.tStart = 50;
start1 = 1;
start2 = 1;
orgy0 = zeros(InfDS.Nsubj,InfDS.NxState);
period = .8;
InfDS.omega = (2*pi/period)^2;


zetaSBP = 3;
beta1 = .5;
beta2 = .5;
lambda21 = .7;
lambda31 = 1.2;
mu1 = 0;
mu2 = 0;
mu3 = 0;
vsigma1 = .5;
vsigma2 = .5;
vsigma3 = .5;
InfDS.G=eye(InfDS.Nx);

truepars = [zetaSBP,beta1,beta2, start1,start2, mu1, mu2, mu3, lambda21,lambda31,vsigma1,vsigma2,vsigma3];


if InfDS.Nb > 0
    vsigmab1 = .5; vsigmab2 = 1; vsigmab3 = 1; v23 = .3;
    
    sigmab = [vsigmab1,0,0;
              0, vsigmab2,v23;
              0, v23, vsigmab3];
    truepars = [truepars,vsigmab1,vsigmab2,vsigmab3,v23];
    InfDS.b= (chol(sigmab)*randn(InfDS.Nb,InfDS.Nsubj))';
    InfDS.b = round2(InfDS.b,10e-4);
    InfDS.Z = eye(InfDS.Nb);%[1;0;0];
    trueb = InfDS.b;
end

%disp(['True parameter values: ',num2str(truepars)]);% 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Simulate data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

InfDS.par = truepars;


full_time = cell(InfDS.Nsubj,1);
full_path = cell(InfDS.Nsubj, InfDS.Nx);
true_DeltaX = cell(InfDS.Nsubj, InfDS.Nx);
InfDS.Deltat = cell(InfDS.Nsubj,1);
InfDS.tobs = cell(InfDS.Nsubj);
InfDS.U1 = zeros(InfDS.Nsubj,InfDS.Nu);
InfDS.H = zeros(InfDS.Ntheta*InfDS.Nsubj, InfDS.Nbetax);
InfDS.lens=numel(InfDS.Nsubj);

InfDS.y0 = [orgy0 repmat(truepars(1:InfDS.Nbetax),InfDS.Nsubj,1)];
PEi = unifrnd(0*ones(1,InfDS.Nsubj),5*ones(1,InfDS.Nsubj));%(5+0)*rand(InfDS.Nsubj,1)-0;
Stressi = unifrnd(0*ones(1,InfDS.Nsubj),5*ones(1,InfDS.Nsubj));%gamrnd(2.5,.5,InfDS.Nsubj,1);

%InfDS.delt is the fixed time step used in numerical solutions of the ODEs
%InfDS.tspan holds a list of time intervals from 0 to InfDS.maxT, with
%increments InfDS.delt
InfDS.maxT = period*3;
InfDS.delt = InfDS.maxT/(InfDS.totalT + InfDS.tStart);
InfDS.delt = max(.005,InfDS.delt/2);
tspan = 0:InfDS.delt:InfDS.maxT;
tspan = round2(tspan,InfDS.delt);
InfDS.tspan = tspan;
minDelt = InfDS.delt;
maxDelt = minDelt;
InfDS.fulldt=cell(InfDS.Nsubj);

for i = 1:InfDS.Nsubj,
    tt = randsample((InfDS.tStart+2):length(tspan),InfDS.totalT-1);
    InfDS.lens(i)=length(tspan);%max(tt);%length including interpolating time points
    InfDS.tobs{i} = [1 sort(tt)-InfDS.tStart];
    InfDS.Deltat{i} = [InfDS.delt; reshape(diff(tspan(InfDS.tobs{i})),length(InfDS.tobs{i})-1,1)];
    full_time{i} =tspan;
    InfDS.fulldt{i}=[InfDS.delt diff(full_time{i})];%diff(full_time{i});%one shorter than
    if (min(InfDS.Deltat{i}) < minDelt), minDelt = min(InfDS.Deltat{i});end
    if (max(InfDS.Deltat{i}) > maxDelt), maxDelt = max(InfDS.Deltat{i});end
    InfDS.U1(i,:) = [Stressi(i) PEi(i)];
    InfDS.H((1+(i-1)*InfDS.Ntheta):(i*InfDS.Ntheta),:) = [1 Stressi(i) PEi(i) 0 0;
                                                          0 0 0 1 0;
                                                          0 0 0 0 1];
    %InfDS.H((1+(i-1)*InfDS.Ntheta):(i*InfDS.Ntheta),:) = [1 Stressi(i) PEi(i)];
end


a =zeros(InfDS.Nx,InfDS.Nsubj,length(tspan));

InfDS.fp=cell(InfDS.Nsubj);
InfDS.Pp=cell(InfDS.Nsubj);


for i=1:InfDS.Nsubj
    dt=InfDS.fulldt{i};
    for t=1:(InfDS.tStart+1)
        %data generation based on RK4, estimation by the second-order Heun method later
        if t==1
        a(:,i,t) = feval(InfDS.odefn, reshape(InfDS.y0(i,:),InfDS.Nx,1),0,InfDS,i,{},1);   
        else
        k1 = feval(InfDS.odefn, a(:,i,t-1),0,InfDS,i,{},0);
        k2 = feval(InfDS.odefn, a(:,i,t-1)+dt(t)/2*k1,0,InfDS,i,{},0);
        k3 = feval(InfDS.odefn, a(:,i,t-1)+dt(t)/2*k2,0,InfDS,i,{},0);
        k4 = feval(InfDS.odefn, a(:,i,t-1)+dt(t)*k3,0,InfDS,i,{},0);
        a(:,i,t) = a(:,i,t-1) + dt(t)/6*(k1 + 2*k2 + 2*k3 + k4);
        end
    end
end%end of i loop

a(1,:,InfDS.tStart+1) = a(1,:,InfDS.tStart+1) - mean(a(1,:,InfDS.tStart+1));
a(2,:,InfDS.tStart+1) = a(2,:,InfDS.tStart+1) - mean(a(2,:,InfDS.tStart+1));

truepars(4:5) = mean(a(1:2,:,InfDS.tStart+1),2);
InfDS.y0(:,1:2) = repmat(truepars(4:5),InfDS.Nsubj,1);
InfDS.y0(:,6:7) = repmat(truepars(4:5),InfDS.Nsubj,1);
trueb(:,2) = a(1,:,InfDS.tStart+1) - truepars(4);
trueb(:,3) = a(2,:,InfDS.tStart+1) - truepars(5);
sigb = cov(trueb(:,2:3));
truepars(15:17) =[diag(sigb)',sigb(1,2)];

%Demean the initial condition
for i=1:InfDS.Nsubj
    dt=InfDS.fulldt{i};
    for t=(InfDS.tStart+1):InfDS.lens(i)
        %data generation based on RK4, estimation by the second-order Heun method later
        if t==InfDS.tStart+1
        a(:,i,t) = feval(InfDS.odefn, reshape(InfDS.y0(i,:),InfDS.Nx,1),0,InfDS,i,{},1);   
        else
        k1 = feval(InfDS.odefn, a(:,i,t-1),0,InfDS,i,{},0);
        k2 = feval(InfDS.odefn, a(:,i,t-1)+dt(t)/2*k1,0,InfDS,i,{},0);
        k3 = feval(InfDS.odefn, a(:,i,t-1)+dt(t)/2*k2,0,InfDS,i,{},0);
        k4 = feval(InfDS.odefn, a(:,i,t-1)+dt(t)*k3,0,InfDS,i,{},0);
        a(:,i,t) = a(:,i,t-1) + dt(t)/6*(k1 + 2*k2 + 2*k3 + k4);
        end
    end
end%end of i loop


%%%%%%%%%%%%%%%%%%%%%%%%
%Keep data after tStart
a = a(:,:,(InfDS.tStart+1):length(tspan));
tspan = tspan((InfDS.tStart+1):length(tspan));
tspan = tspan - min(tspan);
InfDS.tspan = tspan;

for i=1:InfDS.Nsubj
InfDS.lens(i)=length(tspan);%max(tt);%length including interpolating time points
full_time{i} =tspan;
for j=1:InfDS.Nx
     full_path{i,j} = reshape(a(j,i,1:InfDS.lens(i)),InfDS.lens(i),1);
     true_DeltaX{i,j} = [0; diff(full_path{i,j})];%true latent variables
     InfDS.fp{i}(j,:)=full_path{i,j};
     InfDS.dx{i}(j,:)=true_DeltaX{i,j};
end%%%%%%%%%%%%%%%%%%%%%%%%
InfDS.fp{i}=zeros(InfDS.Nx,InfDS.lens(i));
InfDS.dx{i}=zeros(InfDS.lens(i));
end
InfDS.fulltime=full_time;
InfDS.fp0=InfDS.fp;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set up observations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% We start by defining cell arrays:

InfDS.timeDiscrete = cell(InfDS.Nsubj,1);
path_cell = cell(size(InfDS.y0)); %InfDS.Nsubj by dimension of ODE system
path_deltax = cell(size(InfDS.y0));

%truepars = [zetaSBP,beta1,beta2, start1,start2, mu1, mu2, mu3, lambda21,lambda31,vsigma1,vsigma2,vsigma3];


for i = 1:InfDS.Nsubj
    temp = zeros(InfDS.Nx,length(InfDS.tobs{i}));
    temp2 = zeros(InfDS.Nx,length(InfDS.tobs{i}));
    temp2(:,1) = zeros(InfDS.Nx,1);
    InfDS.timeDiscrete{i} = full_time{i}(InfDS.tobs{i});
    for j = 1:InfDS.Nx
        temp(j,:) = full_path{i,j}(InfDS.tobs{i})';
    end
        trueb(i,2) = full_path{i,1}(1)-truepars(4);
        trueb(i,3) = full_path{i,2}(1)-truepars(5);

    
    for t=2:length(InfDS.tobs{i})
        temp2(:,t) =  temp(:,t)-temp(:,t-1);
    end
    for j = 1:InfDS.Nx
        path_cell{i,j} = temp(j,:);
        path_deltax{i,j} = temp2(j,:);
    end
    
end %loop across subjects


%The two should have the same values
if plot_flag == 1
        figure(1)
        clf;
        plot(full_time{1},full_path{1,2},'k-');hold on;
        axis([0 InfDS.maxT -25 25])
        for i=1:InfDS.Nsubj
            plot(InfDS.timeDiscrete{i},path_cell{i,2},'g-.','Linewidth',2);hold on;
            plot(full_time{i},full_path{i,2},'k-','Linewidth',1);hold on;
        end
        hold off;
end

%clear temp temp2 xtemp;
% Finally, we add random observational noise to the 'path' variable.

Ycell = cell(InfDS.Nsubj,1);
Lambda = [1 0;lambda21 0;lambda31 0];
Sigmae = diag([vsigma1, vsigma2, vsigma3]);
mu = [mu1; mu2; mu3];

for i = 1:InfDS.Nsubj
    x = zeros(InfDS.NxState,length(InfDS.tobs{i}));
    for j = 1:InfDS.NxState
        x(j,:) = path_cell{i,j};
    end
    Ymat = zeros(InfDS.Ny, length(path_cell{i,j}));
    e = zeros(InfDS.Ny, length(InfDS.tobs{i}));
    for t=1:length(path_cell{i,j})
        e(:,t) =   chol(Sigmae)*randn(InfDS.Ny,1);
        Ymat(:,t) = mu + Lambda*x(:,t) +  e(:,t);
    end
     Ycell{i} = Ymat;
end

InfDS.Y = Ycell;
InfDS.trueX = path_cell;


    InfDS.allT = zeros(1,InfDS.Nsubj);
    InfDS.allDelta = [];
    for i=1:InfDS.Nsubj
        InfDS.allT(i)=length(InfDS.timeDiscrete{i});
        InfDS.allDelta = [InfDS.allDelta InfDS.Deltat{i}'];
     end
    InfDS.Tfilter = zeros(1,InfDS.Nsubj*max(InfDS.allT));
    for i=1:InfDS.Nsubj
        InfDS.Tfilter(1+(i-1)*max(InfDS.allT):InfDS.allT(i)+(i-1)*max(InfDS.allT)) = 1;
    end
 
InfDS.trueb = trueb;

%Write out true random effects
dlmwrite(filenametrueb,[repmat(batch,InfDS.Nsubj,1), repmat(kk,InfDS.Nsubj,1 ), repmat(trueInit,InfDS.Nsubj,1),...
    reshape(InfDS.trueb,InfDS.Nsubj,InfDS.Nb)],'-append');


%Convert from cell into vectors/matrices
Y = zeros(InfDS.Nsubj*InfDS.totalT, InfDS.Ny+InfDS.Nu+1);
for i=1:InfDS.Nsubj,
Y((1+(i-1)*InfDS.totalT):(i*InfDS.totalT),1) = InfDS.timeDiscrete{i};    
Y((1+(i-1)*InfDS.totalT):(i*InfDS.totalT),2:(InfDS.Ny+1)) = InfDS.Y{i}';
Y((1+(i-1)*InfDS.totalT):(i*InfDS.totalT),(InfDS.Ny+2):(InfDS.Ny+1+InfDS.Nu)) = repmat(InfDS.U1(i,:),InfDS.totalT,1);
end

dlmwrite(filenameY,[repmat(batch,InfDS.Nsubj*InfDS.totalT,1), repmat(kk,InfDS.Nsubj*InfDS.totalT,1 ), ...
    repmat(trueInit,InfDS.Nsubj*InfDS.totalT,1),Y]);


    
% To observe the raw data and the true path, we can plot
if plot_flag == 1
    
        figure(1)
        clf;
        subplot(121)
        p1 = plot(InfDS.timeDiscrete{1}',InfDS.Y{1}(1,:),'r*');hold on;
        p2 = plot(InfDS.timeDiscrete{1}',InfDS.trueX{1,1},'k-');hold on;

        for i=2:min(30,InfDS.Nsubj),%loop across subjects
            plot(InfDS.timeDiscrete{i}',InfDS.Y{i}(1,:),'r*');hold on;
        end
        for i=2:min(30,InfDS.Nsubj),%loop across subjects        
            plot(InfDS.timeDiscrete{i}',InfDS.trueX{i,1},'k-','lineWidth',3);hold on;
        end

           axis([0 InfDS.maxT -5 5.5]);
            xlabel('\fontsize{16} t_{i,j}','FontName','Arial');
            ylabel('\fontsize{16} x_{i1}(t_{i,j}) and y_{i1}(t_{i,j})','FontName','Arial');
            title('\fontsize{16} (A) Time-based dynamics','FontName','Arial');
             %plot(InfDS.timeDiscrete{j,1},repmat(mu1,length(path_cell{j,1}),1),'g-','lineWidth',3);
            pp=legend([p1 p2],'Observed','True');
             hold off;

            
        subplot(122)
          axis([-5 5 -5 5]);
         %for i=1:min(30,InfDS.Nsubj),%loop across subjects
         %   plot(InfDS.Y{i}(1,:),InfDS.Y{i}(2,:),'r*');hold on;
         %end
        for i=1:min(30,InfDS.Nsubj),%loop across subjects        
            plot(InfDS.trueX{i,1},InfDS.trueX{i,2},'k-','lineWidth',1);hold on;
        end
            xlabel('\fontsize{14} x_{i1}(t_{i,j})','FontName','Arial');
            ylabel('\fontsize{14} x_{i2}(t_{i,j})','FontName','Arial');
            title('\fontsize{16} (B) True latent variable scores','FontName','Arial');
            %plot(InfDS.timeDiscrete{j,1},repmat(mu1,length(path_cell{j,1}),1),'g-','lineWidth',3);
            hold off;
 
set(pp, 'Box', 'off','Color', 'none','FontSize',16,'FontName','Arial','Location','Best')            
end

end